# coffeelint: disable=max_line_length

exec = require "child_process"
  .exec
_ = require "lodash"
debug = require("../../../shared_modules/logger").debug

async = require 'async'
dns =require "dns"
validateip = require 'validate-ip'

Producer = require "../../ProducerClass"

class Scansipdestinations extends Producer
  constructor: ->
    @isRunning = false
    super "scansipdestinations"
    handleConfig = (err, status, res) =>
      if err
        debug err
        return

      res = res or []

      destinations = []

      _.each res, (channel, index) ->
        if not channel.bool_CfgValid #return if channel is deactivated
          debug "channel #{index} is deactivated"
          return
        if channel.struct_GateParams and channel.struct_GateParams.struct_CallProcTable
          addDestination = (rule) ->
            if not _.isObject(rule) or rule.string_sipDefaultGatewayIpAddress is ""
              return
            #return if the rule is connected to a messaging server,
            #which means that a different port (the one from the Dchannel) is used
            if rule.string_Destination is "FMS"
              return

            destination =
              ilidName: "#{channel.string_IlidSlot}.#{channel.string_IlidIndex}"
              address: rule.string_sipDefaultGatewayIpAddress,
              proto: rule.string_sipDefaultGatewayIpMode.toLowerCase(),
              port: rule.string_sipDefaultGatewayIpPort,
              info: rule.string_Description,
              disabled: rule.string_disabled,
              resolved: null

            destinations.push(destination)

          #has multiple rules
          if channel.struct_GateParams.struct_CallProcTable[0]
            _.each channel.struct_GateParams.struct_CallProcTable, (rule) ->
              addDestination rule
          #has just one rule
          else
            addDestination channel.struct_GateParams.struct_CallProcTable

      async.eachOf destinations, (host, key, cb) =>
        debug "scanning", host.address
        ips = []
        if validateip host.address
          ips.push host.address

        async.series [
          (v4resolved) ->
            if validateip host.address
              v4resolved()
              return

            dns.resolve4 host.address, (err, result) ->
              debug "v4 #{JSON.stringify result}"
              ips = _.concat ips, result
              v4resolved()
          (v6resolved) ->
            if validateip host.address
              v6resolved()
              return

            dns.resolve6 host.address, (err, result) ->
              debug "v6 #{JSON.stringify result}"
              ips = _.concat ips, result
              v6resolved()

          (omgprobed) ->
            exec "./omgprobe.lua #{host.address} #{host.port} #{host.proto} --json", {cwd: __dirname + "/lua/"}, (error, stdout) -> #, stderr
              try
                stdout = JSON.parse(stdout)
              catch e
                console.log "Could not parse output of omgprobe.lua; #{e}"

              destinations[key].resolved = stdout.resolved
              omgprobed()

        ], =>
          ips = _.uniq _.compact ips
          async.each ips, (ip, ipTested) ->
            exec "LC_ALL=C /usr/bin/sudo /usr/bin/nmap -sT -Pn #{if ip.match(':') then '-6' else ''} #{ip} -p #{host.port} 2>/dev/null | grep ^#{host.port}/tcp", (err, stdout) ->
              debug 'err', err if err
              # remove multiple spaces
              while stdout.match /\ \ /g
                stdout = stdout.replace /\ \ /g, " "
              # debug "nmap result:", stdout
              states = []
              if stdout.match("open") then states.push "open"
              if stdout.match("closed") then states.push "closed"
              if stdout.match("filtered") then states.push "filtered"
              if stdout.match("unfiltered") then states.push "unfiltered"

              states.push stdout.split(" ")[2] if stdout.split(" ")[2]

              scanResult = states.join "|"
              destinations[key].resolved[ip]?.portProbe = scanResult
              ipTested()
          , =>
            @sendData destinations
            cb()

    @start = =>
      @sendData {}, true
      @debug "starting"

      requestObject =
        producer: "configurator"
        action: "get"
        args: ["config.lua struct_DChannel"]

      @requestData requestObject, handleConfig

    @stop = =>
      @isRunning = false


module.exports = new Scansipdestinations()
